function ray_trace_boss
%RAYTRACE  Run a demo of a trajectory of an acoustic ray
%based on Matlabs' ballode.m demo program
%y=r,z,theta
%   E. Boss, UMaine, 9/2012

%options for the ODE solver
refine = 4;
options = odeset('Events',@events,'OutputFcn',@odeplot,'OutputSel',1,...
                 'Refine',refine);

tstart = 0;  %initial time [s]
tfinal = 30; %final propagation time [s]

rstart = 0; % initial horizontal position [m]
zstart = -3000; % initial depth [m]
qstart = 3; %initial angle [degrees]

min_z = 0; %minimal depth [m]
max_z=-5000; %maximal depth [m]

y0 = [rstart; zstart; qstart*pi/180]; %initial condition

figure;
set(gca,'ylim',[max_z 0]);
box on
hold on;

tout = tstart;
yout = y0.';
teout = [];
yeout = [];
ieout = [];
for i = 1:10%number of terminal events
  % Solve until the first terminal event.
  [t,y,te,ye,ie] = ode45(@f,[tstart tfinal],y0,options);  
  if ~ishold
    hold on
  end
  % Accumulate output.  This could be passed out as output arguments.
  nt = length(t);
  tout = [tout; t(2:nt)]
  yout = [yout; y(2:nt,:)]
  teout = [teout; te];          % Events at tstart are never reported. 
  yeout = [yeout; ye];
  ieout = [ieout; ie];
  
  ud = get(gcf,'UserData');
  if ud.stop
    break;
  end
  
  % Set the new initial conditions
  y0(1) = y(nt,1);
  y0(2) = y(nt,2);
  y0(3) = -y(nt,3);%reflection about the horizontal

  % A good guess of a valid first timestep is the length of the last valid
  % timestep, so use it for faster computation.  'refine' is 4 by default.
  options = odeset(options,'InitialStep',t(nt)-t(nt-refine),...
                           'MaxStep',t(nt)-t(1));

  tstart = t(nt);
  if tstart==tfinal
      break
  end
end

plot(yout(:,1),yout(:,2),'ro')
xlabel('distance');
ylabel('depth');
title('Ray trajectory and the events');
hold off
odeplot([],[],'done');
[c, dcdz] = s_spd([0:-5:-5000]);
figure;plot(c,[0:-5:-5000]);

% --------------------------------------------------------------------------
%heart of the integration routing. Computes gradients
function dydt = f(t,y)    
[c, dc_dz] = s_spd(y(2));
dydt = [c/sec(y(3));
    tan(y(3))*c/sec(y(3)); 
-dc_dz/sec(y(3))];
    
% --------------------------------------------------------------------------

%function to providing sound speed and its gradient.
function [c, dcdz] = s_spd(z)
zz = -[0:1:5000];
cc = 1520 + [zz*.05];
cc (751:end) = cc(750) + -(zz(751:end)-zz(750))*.014;
A=diff(cc);
dcdz=-[A(1),A,A(length(A))];
z_dc_dz=-[0,0.5:1:4999.5,5000];

c=interp1(zz,cc,z,'linear','extrap');
dcdz=interp1(z_dc_dz,dcdz,z,'linear','extrap');

% --------------------------------------------------------------------------

% Locates the times when height passes through zero in any direction
function [value,isterminal,direction] = events(t,y)
value = y(2)*(y(2)+5000);     % detect depth = 0/-5000
isterminal = 1;   % stop the integration
direction = []; %any direction

